From 7cace648c0adba4272fdf820f9ee8a471fb788fc Mon Sep 17 00:00:00 2001
From: Deomid Ryabkov <rojer@cesanta.com>
Date: Mon, 8 Aug 2016 12:24:01 +0100
Subject: [PATCH] Improve cookie auth example

Use templated index page and SSI calls instead of hard-coded html

PUBLISHED_FROM=eade9dfe283952ed5465cd0ff1924f61a14dea0c
---
 examples/cookie_auth/cookie_auth.c | 71 +++++++++++++++++-------------
 examples/cookie_auth/index.shtml   | 34 ++++++++++++++
 examples/cookie_auth/login.html    |  4 +-
 3 files changed, 77 insertions(+), 32 deletions(-)
 create mode 100644 examples/cookie_auth/index.shtml

diff --git a/examples/cookie_auth/index.shtml b/examples/cookie_auth/index.shtml
new file mode 100644
index 000000000..32bb414ab
--- /dev/null
+++ b/examples/cookie_auth/index.shtml
@@ -0,0 +1,34 @@
+<!DOCTYPE html>
+<html lang="en">
+<head>
+  <meta charset="utf-8" />
+  <title>Main - Mongoose cookie_auth</title>
+  <meta name="viewport" content="width=device-width, initial-scale=1" />
+  <style type="text/css">
+    * { outline: none; }
+    body {
+      background-color: #789; margin: 0;
+      padding: 0; font: 16px/1.4 Helvetica, Arial, sans-serif;
+      font: 16px/1.4 Helvetica, Arial, sans-serif;
+    }
+    div.content {
+      width: 600px; margin: 2em auto; padding: 20px 50px;
+      background-color: #fff; border-radius: 1em;
+    }
+    label { display: inline-block; min-width: 7em; }
+    input { border: 1px solid #ccc; padding: 0.2em; }
+    a:link, a:visited { color: #69c; text-decoration: none; }
+    @media (max-width: 700px) {
+      body { background-color: #fff; }
+      div.content { width: auto; margin: 0 auto; padding: 1em; }
+    }
+</style>
+</head>
+<body>
+  <div class="content">
+    <h1>Hello, <!--#call user -->!</h1>
+    <p>Your lucky number is <!--#call lucky_number -->.</p>
+    <p><a href="/logout">Log out</a>
+  </div>
+</body>
+</html>
diff --git a/examples/cookie_auth/login.html b/examples/cookie_auth/login.html
index c6ec5ac06..0b49a621e 100644
--- a/examples/cookie_auth/login.html
+++ b/examples/cookie_auth/login.html
@@ -2,7 +2,7 @@
 <html lang="en">
 <head>
   <meta charset="utf-8" />
-  <title>Please log in</title>
+  <title>Log in - Mongoose cookie_auth</title>
   <meta name="viewport" content="width=device-width, initial-scale=1" />
   <style type="text/css">
     * { outline: none; }
@@ -12,7 +12,7 @@
       font: 16px/1.4 Helvetica, Arial, sans-serif;
     }
     div.content {
-      width: 800px; margin: 2em auto; padding: 20px 50px;
+      width: 600px; margin: 2em auto; padding: 20px 50px;
       background-color: #fff; border-radius: 1em;
     }
     label { display: inline-block; min-width: 7em; }
diff --git a/examples/cookie_auth/cookie_auth.c b/examples/cookie_auth/cookie_auth.c
index 5397e169d..b2484258b 100644
--- a/examples/cookie_auth/cookie_auth.c
+++ b/examples/cookie_auth/cookie_auth.c
@@ -197,37 +197,48 @@ void check_sessions() {
 
 /* Main event handler. */
 static void ev_handler(struct mg_connection *nc, int ev, void *p) {
-  /* Perform session maintenance. */
-  if (ev == MG_EV_TIMER) {
-    check_sessions();
-    mg_set_timer(nc, mg_time() + SESSION_CHECK_INTERVAL);
-    return;
-  }
-  if (ev != MG_EV_HTTP_REQUEST) return;
-
-  nc->flags |= MG_F_SEND_AND_CLOSE;
-  struct http_message *hm = (struct http_message *) p;
-  struct session *s = get_session(hm);
-  /* Ask the user to log in if they did not present a valid cookie. */
-  if (s == NULL) {
-    mg_printf(nc,
-              "HTTP/1.0 302 Found\r\n"
-              "Location: /login.html\r\n"
-              "\r\n"
-              "Please log in");
-    return;
+  switch (ev) {
+    case MG_EV_HTTP_REQUEST: {
+      struct http_message *hm = (struct http_message *) p;
+      struct session *s = get_session(hm);
+      /* Ask the user to log in if they did not present a valid cookie. */
+      if (s == NULL) {
+        mg_printf(nc,
+                  "HTTP/1.0 302 Found\r\n"
+                  "Location: /login.html\r\n"
+                  "\r\n"
+                  "Please log in");
+        nc->flags |= MG_F_SEND_AND_CLOSE;
+        break;
+      }
+      /*
+       * Serve the page that was requested.
+       * Save session in user_data for use by SSI calls.
+       */
+      fprintf(stderr, "%s (sid %" INT64_X_FMT ") requested %.*s\n", s->user,
+              s->id, (int) hm->uri.len, hm->uri.p);
+      nc->user_data = s;
+      mg_serve_http(nc, (struct http_message *) p, s_http_server_opts);
+      break;
+    }
+    case MG_EV_SSI_CALL: {
+      /* Expand variables in a page by using session data. */
+      const char *var = (const char *) p;
+      const struct session *s = (const struct session *) nc->user_data;
+      if (strcmp(var, "user") == 0) {
+        mg_printf_html_escape(nc, "%s", s->user);
+      } else if (strcmp(var, "lucky_number") == 0) {
+        mg_printf_html_escape(nc, "%d", s->lucky_number);
+      }
+      break;
+    }
+    case MG_EV_TIMER: {
+      /* Perform session maintenance. */
+      check_sessions();
+      mg_set_timer(nc, mg_time() + SESSION_CHECK_INTERVAL);
+      break;
+    }
   }
-  /* Application logic that uses session data goes here. */
-  fprintf(stderr, "%s (sid %" INT64_X_FMT ") requested %.*s\n", s->user, s->id,
-          (int) hm->uri.len, hm->uri.p);
-  mg_printf(nc,
-            "HTTP/1.0 200 Ok\r\n"
-            "COntent-Type: text/html\r\n"
-            "\r\n"
-            "<h1>Hello, %s!</h1>\r\n"
-            "<p>Your lucky number is %d.</p>\r\n"
-            "<p><a href=/logout>Log out</a>",
-            s->user, s->lucky_number);
 }
 
 int main(void) {
-- 
GitLab